/*

*/

import server.*;

import java.time.format.DateTimeFormatter;
import java.time.OffsetDateTime;
import java.time.ZoneOffset;

import java.text.DateFormat;
import java.util.Date;
import java.util.TimeZone;

import java.util.Scanner;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.IOException;

/**
   This is an example of how you can use the HttpApplicationServer
   framework in the server package to implement a very simple
   web application.

   This application expects the name of a city and then looks up
   that city in Java's built-in list of time zones. If the city
   is found in the list, then the current time in that city is
   returned.
*/
public class TimeZoneServer extends HttpMethodsAdaptor
{
   private final DateFormat timeFormatter = DateFormat.getTimeInstance();
   private final String[] timeZoneIds = TimeZone.getAvailableIDs();

   /**
      Give this sample server an implementation of GET.
   */
   @Override
   public void doGet(HttpRequest req, HttpResponse res)
   {
      System.out.println("SERVER: GET addition method:");

      if ( ! req.resourceName.equals("/timeZoneLookUp") )
      {
         doStaticGet(req, res);
         return;
      }

      // Get the URL's query string from the req object.
      final String query = req.queryString;
      // Parse it.
      final int cityIndex = query.indexOf("=") + 1;
      final String cityName = query.substring(cityIndex);

      // Look up the city name in Java's built-in list of time zones.
      TimeZone timeZone = null;
      for (int i = 0; i < timeZoneIds.length; ++i)
      {
         final String timeZoneId = timeZoneIds[i];
         final int timeZoneCityIndex = timeZoneId.lastIndexOf('/') + 1;
         final String timeZoneCity = timeZoneId.substring(timeZoneCityIndex);
         if (timeZoneCity.replace('_', ' ').equals(cityName))
         {
            // Found.
            timeZone = TimeZone.getTimeZone(timeZoneId);
            break;
         }
      }

      // Build the response body.
      final File file;
      if (timeZone != null) // if the city name was found
      {
         file = new File(server.serverRoot + "/next.html"); // result template page
      }
      else // if the city name was not found
      {
         file = new File(server.serverRoot + "/error.html"); // error tempalte page
      }

      String body = "";
      try (final Scanner inFile = new Scanner(file))
      {
         while (inFile.hasNextLine()) // Read the whole html template page.
         {
             body += inFile.nextLine() + "\n";
         }
      }
      catch (FileNotFoundException e)
      {
         System.out.println("SERVER: Cannot find file: " + file);
         System.exit(-1);
      }

      // Replace the name template parameter with the name of the city.
      body = body.replaceFirst("###timeZoneCity###", cityName);

      // Replace the time template parameter with the city's time.
      if (timeZone != null) // if the city name was found
      {
         timeFormatter.setTimeZone(timeZone);
         final Date time = new Date();
         final String timeString = timeFormatter.format(time);
         body = body.replaceFirst("###cityTimeZone###", timeString);
      }

      // Add the response line.
      res.setResponseLine("HTTP/1.1 200 OK");

      // Add the response headers.
      final String date = OffsetDateTime.now(ZoneOffset.UTC)
                            .format(DateTimeFormatter.RFC_1123_DATE_TIME);
      res.addHeader("Date: " + date);
      res.addHeader("Server: TimeZoneServer.java");
      res.addHeader("Connection: close");
      res.addHeader("Content-type: text/html");
      res.addHeader("Content-Length: " + body.length());

      // Add the entity body.
      res.setEntityBody( body.getBytes() );

      return;
   }


   /**
      Send a static resource to the client.
   */
   public void doStaticGet(HttpRequest req,
                           HttpResponse res)
   {
      System.out.println("SERVER: doStaticGet: ");

      File file = new File(server.serverRoot + req.resourceName);
      if ( ! file.exists() || ! file.canRead() )
      {
         if (! file.exists())
         {
            sendErrorResponse(req, res,
                "404", "Not Found",
                "The requested URL "+req.resourceName+" was not found on this server.");
         }
         else
         {
            sendErrorResponse(req, res,
                "403", "Forbidden",
                "The requested URL "+req.resourceName+" is not readable.");
         }
         return;  // Finished with this request.
      }
      if ( file.isDirectory() )
      {
         // Check for existence of an index.html file.
         final File file2 = new File(server.serverRoot + req.resourceName + "index.html");
         if ( file2.exists() && file2.canRead() )
         {
            req.resourceName = req.resourceName + "index.html";
            file = new File(server.serverRoot + req.resourceName);
         }
         else
         {
            sendErrorResponse(req, res,
                "400", "Bad Request",
                "The requested URL, "+req.resourceName+", does not contain a file name.");
            return;  // Finished with this request.
         }
      }

      final int numOfBytes = (int)file.length();
      try
      {
         final FileInputStream inFile = new FileInputStream(file);
         final byte[] fileInBytes = new byte[numOfBytes];
         inFile.read(fileInBytes);

         // Add the response line.
         res.setResponseLine("HTTP/1.1 200 OK");
         // Add the response headers.
         final String date = OffsetDateTime.now(ZoneOffset.UTC)
                               .format(DateTimeFormatter.RFC_1123_DATE_TIME);
         res.addHeader("Date: " + date);
         res.addHeader("Server: HttpApplicationServer.java");
         res.addHeader("Connection: close");
         final String contentType;
         if ( req.resourceName.endsWith(".html") || req.resourceName.endsWith(".htm") )
            contentType = "text/html";
         else if ( req.resourceName.endsWith(".css") )
            contentType = "text/css";
         else if ( req.resourceName.endsWith(".js") )
            contentType = "application/javascript";
         else if ( req.resourceName.endsWith(".jpg") || req.resourceName.endsWith(".jpeg") )
             contentType = "image/jpeg";
         else if ( req.resourceName.endsWith(".gif") )
            contentType = "image/gif";
         else if ( req.resourceName.endsWith(".png") )
             contentType = "image/png";
         else  // default content type
            contentType = "text/plain";
         res.addHeader("Content-Type: " + contentType);
         res.addHeader("Content-Length: " + numOfBytes);

         // Add the entity body.
         if ( ! req.requestMethod.equals("HEAD") )
         {
            res.entityBody = fileInBytes;
         }
      }
      catch (FileNotFoundException e)
      {
         System.out.println("SERVER: doStaticGet: Unable to find file.");
         System.out.println( e );
      }
      catch (IOException e)
      {
         System.out.println("SERVER: doStaticGet: Unable to read file.");
         System.out.println( e );
      }
   }


   /**
      The main() method configures the server using
      the httpserver.properties file and command-line
      arguments.
   */
   public static void main(String[] args)
   {
      String serverRoot = "public_html"; // default value
      int portNumber = 8080;             // default value

      if (0 < args.length)
      {
         portNumber = Integer.parseInt(args[0]);
      }
      if (1 < args.length)
      {
         serverRoot = args[1];
      }

      // Instantiate and start up an instance of the server.
      new HttpApplicationServer(portNumber,
                                serverRoot,
                                new TimeZoneServer()); // HTTP methods
   }
}
